# flow
A wave module solving the Full Potential Equation

![](/dox/flow.png)

## Main features
* Meshes generated using gmsh
  - [x] 2D computations (Line2 and Tri3 elements)
  - [x] 3D computations (Tri3 and Tetra4 elements)
* Solvers
  - [x] nonlinear Newton-Raphson with analytic tangent matrix and quadratic or Bank&Rose line search
  - [x] linear Intel MKL Pardiso or Eigen's GMRES or MUMPS
* Post-processing
  - [x] data extracted at nodes readily available in python and saved to disk
  - [x] ascii, gmsh or vtk data format
* Various
  - [x] multi-isolated lifting surfaces (e.g. wing/tail) generated using geoGen
  - [x] Kutta condition enforced through direct matrix maniplation
  - [x] adaptive artificial viscosity computed with best-upstream-alignement strategy
  - [x] mesh deformation using linear elasticity laws and aeroelastic computation using CUPyDO

## Sub-modules
* [tests](flow/tests): simple tests to be run in battery checking the basic functionnalities of the solver
* [validation](flow/validation): large-scale tests used to validate the solver results
* [benchmark](flow/benchmark): large-scale tests used to monitor the performance
* [cases](flow/cases) : sample configuration files meant to be run with scripts, user-defined case should be derived from these
* [scripts](flow/scripts): various scripts for running the solver automatically
