/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WF1EL_H
#define WF1EL_H

#include "flow.h"
#include "wFct1.h"
#include <vector>
#include <Eigen/Dense>

namespace flow
{

/**
 * @brief Freestream velocity
 * @authors Adrien Crovato
 */
class FLOW_API F1ElVi : public tbox::Fct1
{
    int nDim;           ///< number of dimensions
    Eigen::Vector3d v;  ///< freestream velocity vector
    Eigen::Vector3d dv; ///< freestream velocity vector gradient (wrt alpha)

public:
    F1ElVi(int _nDim, double alpha, double beta = 0);
    void update(double alpha, double beta = 0);

#ifndef SWIG
    virtual Eigen::Vector3d eval(tbox::Element const &e,
                                 std::vector<double> const &u,
                                 size_t k) const override;
    virtual Eigen::Vector3d evalD(tbox::Element const &e,
                                  std::vector<double> const &u,
                                  size_t k) const override;
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace flow

#endif //WF1EL_H
