/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WNEWTON_H
#define WNEWTON_H

#include "flow.h"
#include "wSolver.h"

#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Sparse>

namespace flow
{

/**
 * @brief Newton solver
 * @authors Adrien Crovato
 */
class FLOW_API Newton : public Solver
{
    friend class Adjoint;
    friend class FpeLSFunction;

public:
    double lsTol;   ///< tolerance for line search
    int maxLsIt;    ///< max number of line search
    double avThrsh; ///< residual threshold to deacrease artificial viscosity

private:
    double mCOv; ///< variable cut-off Mach number
    double muCv; ///< variable artificial viscosity scaling factor

public:
    Newton(std::shared_ptr<Problem> _pbl, std::shared_ptr<tbox::LinearSolver> _linsol);

    virtual bool run() override;

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void buildJac(Eigen::SparseMatrix<double, Eigen::RowMajor> &J);
    void buildRes(Eigen::Map<Eigen::VectorXd> &R);
    void findUpwd();
};

} // namespace flow

#endif //WNEWTON_H
