#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# A NETTOYER

import heat as h
import heat.utils as u
import tbox
import tbox.gmsh as gmsh
import fwk
from fwk.testing import *

class MyF(tbox.Fct2XYZ):
    def __init__(self, k1, k2):
        tbox.Fct2XYZ.__init__(self)
        self.k1 = k1
        self.k2 = k2
        
    def eval(self, pt, k, fake):
        x = pt[0]
        y = pt[1]
        L = 1.
        import math
        v = (self.k1+self.k2) / 2 + abs(self.k2-self.k1)/2*math.sin(2*math.pi/L*4*(x+y/2))
        k.resize(2, 2)
        k[0,0] = k[1,1] = v;
        """
        return (self.k1+self.k2)/2+ abs(self.k2-self.k1)/2*math.sin(2*math.pi/L*4*x)*math.sin(2*math.pi/L*4*y)

        if ((int(x/L*10))%2) ^ ((int(y/L*10))%2) ==1:
            return self.k1
        else:
            return self.k2
        """

def main():

    chrono = fwk.Timer(); chrono.start()
    from fwk.wutils import parseargs
    args=parseargs()

    # main problem
    msh = gmsh.MeshLoader("../../models/rect.geo", __file__).execute()
    gmshWriter = tbox.GmshExport(msh)
    pbl = h.Problem(msh)

    myk = h.CompiledFct1b(1., 1000.)
    pbl.add(h.Medium(msh, "S1", myk))

    TM   = 0.0
    dTxM = 0.3
    dTyM = 0.2
    p = h.Periodic(pbl, TM, dTxM, dTyM)
    pbl.add(p)
    
    solver = u.solve(pbl, gmshWriter)

    q1i = p.qM[0]
    q2i = p.qM[1]        
    
    # sensitivity

    p.reset(TM, 1.0, 0.0)
    solver.start(gmshWriter)
    K11 = p.qM[0]
    K21 = p.qM[1]
    if not args.nogui:     
        from heat.viewer import GUI
        GUI().open(pbl.msh.name)
        
    p.reset(TM, 0.0, 1.0)
    solver.start(gmshWriter)
    K12 = p.qM[0]
    K22 = p.qM[1]
    if not args.nogui:  
        from heat.viewer import GUI
        GUI().open(pbl.msh.name)
        
    # try to calculate the first flux with the computed sensitivity matrix

    q1 = K11*dTxM+K12*dTyM
    q2 = K21*dTxM+K22*dTyM
    print()
    print("K=[[", -K11, ",", -K12,"]")
    print("   [", -K21, ",", -K22,"]]")
    print("q  =", q1, q2)
    print("qi =", q1i, q2i)
    print()
    chrono.stop()
    print(chrono)

    tests = CTests()
    tests.add(CTest('q-qi (x)', abs(q1-q1i), 0.0, 1e-6))
    tests.add(CTest('q-qi (y)', abs(q2-q2i), 0.0, 1e-6))
    tests.run()
    

if __name__ == "__main__":
    main()
