#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import numpy as np
from matplotlib import cm
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import rc
from matplotlib.ticker import MultipleLocator
import os

def scatter(xi,sol,d,fig,ax,display,m_i):
    plt.figure(fig.number)
    if d == 1:
        plt.scatter(xi[0], sol)
    elif d == 2:
        ax.scatter(xi[0],xi[1], sol)
    elif d == 3:
        ax.scatter(xi[0],xi[1], xi[2])

    if display == True:
        plt.draw()
        os.chdir('images')
        plt.savefig('training_point' + str(m_i+1) + '.png', bbox_inches = 'tight')
        os.chdir('..')

def scatters(xi,sol,d,fig,ax,display,m_i):
    plt.figure(fig.number)
    if d == 2:
        ax.scatter(xi[:,0],xi[:,1], sol[:])
    elif d == 3:
        c = np.ones(len(xi[:,0]))
        s = ax.scatter(xi[:,0],xi[:,1], xi[:,2],c=sol, cmap=cm.jet, lw = 0)
        s.set_edgecolors = s.set_facecolors = lambda *args:None

    if display == True:
        plt.draw()
        os.chdir('images')
        plt.savefig('training_point' + str(m_i+1) + '.png', bbox_inches = 'tight')
        os.chdir('..')

def surf(X, Y, surrogate, sol, fixed_value, fig, ax, current_title, keep, save, new_title, name,d):
    if d == 1:
        surf = ax.plot(X,surrogate)
        current_title.set_text(new_title)
        ax.set_ylim(np.amin(sol)-1.,np.amax(sol)+1.)
    elif d == 2:
        surf = ax.plot_surface(X, Y, surrogate, rstride=1, cstride=1, cmap=cm.jet, alpha=0.4,
                           linewidth=0, antialiased=False)
        current_title.set_text(new_title)
        ax.set_zlim3d(np.amin(sol),np.amax(sol))
    '''
    elif d == 3:
        tmp = fixed_value[0]*np.ones(X.shape)
        surrogate = surrogate-surrogate.min()
        surrogate = surrogate/surrogate.max()
        surf = ax.plot_surface(X, Y, tmp, rstride=1, cstride=1, facecolors=cm.jet(surrogate), alpha=0.4,
                           linewidth=0, antialiased=False)
        current_title.set_text(new_title)
    '''
    if save == True:
        os.chdir('images')
        #plt.savefig(name, bbox_inches = 'tight')
        plt.savefig(name,
            #This is simple recomendation for publication plots
            dpi=1000,
            # Plot will be occupy a maximum of available space
            bbox_inches='tight',
            )
        os.chdir('..')
    if keep == False:
        surf.remove()
        del surf


def surf4d(X, Y, Z, surrogate, c_min, c_max, fig, ax, current_title, keep, save, new_title, name,d):
    if d == 3:
        surrogate = surrogate-c_min
        surrogate = surrogate / (c_max-c_min)
        surf = ax.plot_surface(X, Y, Z, rstride=1, cstride=1, facecolors=cm.jet(surrogate), alpha=0.5,
                           linewidth=0, antialiased=False)
        current_title.set_text(new_title)

    if save == True:
        os.chdir('images')
        #plt.savefig(name, bbox_inches = 'tight')
        plt.savefig(name,
            #This is simple recomendation for publication plots
            dpi=1000,
            # Plot will be occupy a maximum of available space
            bbox_inches='tight',
            )
        os.chdir('..')
    if keep == False:
        surf.remove()
        del surf


def init(d):
    fig = plt.figure()
    if d == 1:
        ax = fig.gca()
    elif d == 2:
        ax = fig.gca(projection='3d')
        ax.set_xlabel('$T [$C$]$')
        ax.set_ylabel('$H_1 [$mm$]$')
        ax.set_zlabel('max $\sigma_{VM} [$MPa$]$')
        ax.view_init(elev=20., azim=-140.)
    plt.ion()

    current_title = plt.title("Training points")

    #-------------------------
    rc('font',size=16)
    rc('font',family='serif')
    rc('axes',labelsize=16)

    #Direct input
    #plt.rcParams['text.latex.preamble']=[r"\usepackage{lmodern}"]
    #Options
    #rc('text',usetex = True)
    #params = {'text.usetex' : True,
    #          'font.size' : 11,
    #          'font.family' : 'lmodern',
    #          'text.latex.unicode': True,
    #          }
    #plt.rcParams.update(params)

    #-------------------------
    return fig, ax, current_title


def init2(d, images_parameters):
    fig = plt.figure()
    if d == 1:
        ax = fig.gca()
    elif d == 2:
        ax = fig.gca(projection='3d')
        ax.set_xlabel(images_parameters['xlabel'])
        ax.set_ylabel(images_parameters['ylabel'])
        ax.set_zlabel(images_parameters['zlabel'])
        ax.view_init(elev=20., azim=-140.)
    elif d == 3:
        ax = fig.gca(projection='3d')
        ax.set_xlabel(images_parameters['xlabel'])
        ax.set_ylabel(images_parameters['ylabel'])
        ax.set_zlabel(images_parameters['zlabel'])

        ax.view_init(elev=20., azim=-140.)
    plt.ion()

    current_title = plt.title("Training points")

    #-------------------------
    #rc('font',size=16)
    #rc('font',family='serif')
    #rc('axes',labelsize=16)


    if d == 3:
        [t.set_va('center') for t in ax1.get_yticklabels()]
        [t.set_ha('left') for t in ax1.get_yticklabels()]
        [t.set_va('center') for t in ax1.get_xticklabels()]
        [t.set_ha('right') for t in ax1.get_xticklabels()]
        [t.set_va('center') for t in ax1.get_zticklabels()]
        [t.set_ha('left') for t in ax1.get_zticklabels()]

        ax.xaxis._axinfo['tick']['inward_factor'] = 0
        ax.xaxis._axinfo['tick']['outward_factor'] = 0.4
        ax.yaxis._axinfo['tick']['inward_factor'] = 0
        ax.yaxis._axinfo['tick']['outward_factor'] = 0.4
        ax.zaxis._axinfo['tick']['inward_factor'] = 0
        ax.zaxis._axinfo['tick']['outward_factor'] = 0.4
        ax.zaxis._axinfo['tick']['outward_factor'] = 0.4

        ax.xaxis.set_major_locator(MultipleLocator(5))
        ax.yaxis.set_major_locator(MultipleLocator(5))
        ax.zaxis.set_major_locator(MultipleLocator(0.01))
    #Direct input
    #plt.rcParams['text.latex.preamble']=[r"\usepackage{lmodern}"]
    #Options
    #rc('text',usetex = True)
    #params = {'text.usetex' : True,
    #          'font.size' : 11,
    #          'font.family' : 'lmodern',
    #          'text.latex.unicode': True,
    #          }
    #plt.rcParams.update(params)

    #-------------------------
    return fig, ax, current_title

def finish():
    plt.ioff()
    plt.show()

def SVD(n_max,s):
    fig = plt.figure()
    plt.semilogy(list(range(1,n_max+1)),s,'ko')
    ax = fig.gca()
    ax.set_xlabel('$i [-]$')
    ax.set_ylabel('$\sigma_i [-]$')
    os.chdir('images')
    #plt.savefig('SVD.png', bbox_inches = 'tight')
    plt.savefig("SVD.png",
        #This is simple recomendation for publication plots
        dpi=1000,
        # Plot will be occupy a maximum of available space
        bbox_inches='tight',
        )
    os.chdir('..')

def plot2D(x,y,xlabel,ylabel,name):
    fig = plt.figure()
    plt.plot(x,y,'ko')
    ax = fig.gca()
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    os.chdir('images')
    #plt.savefig('SVD.png', bbox_inches = 'tight')
    plt.savefig(name,
        #This is simple recomendation for publication plots
        dpi=1000,
        # Plot will be occupy a maximum of available space
        bbox_inches='tight',
        )
    os.chdir('..')

    return fig, ax

def add2D(x,y,param,name):
    plt.plot(x,y,param)
    os.chdir('images')
    #plt.savefig('SVD.png', bbox_inches = 'tight')
    plt.savefig(name,
        #This is simple recomendation for publication plots
        dpi=1000,
        # Plot will be occupy a maximum of available space
        bbox_inches='tight',
        )
    os.chdir('..')
