/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wResults.h"

using namespace tbox;

Results::Results()
{
    nt = 0;
    time = 0.0;
}

void Results::write(std::ostream &out) const
{
    out << "Results (nt=" << nt << ", time=" << time << ")\n";

    if (!scalars_at_nodes.empty())
    {
        out << scalars_at_nodes.size() << " scalars at nodes: ";
        for (auto &p : scalars_at_nodes)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
    if (!scalars_at_elems.empty())
    {
        out << scalars_at_elems.size() << " scalars at elems: ";
        for (auto &p : scalars_at_elems)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
    if (!vectors_at_nodes.empty())
    {
        out << vectors_at_nodes.size() << " vectors at nodes: ";
        for (auto &p : vectors_at_nodes)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
    if (!vectors_at_elems.empty())
    {
        out << vectors_at_elems.size() << " vectors at elems: ";
        for (auto &p : vectors_at_elems)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
    if (!tensors_at_nodes.empty())
    {
        out << tensors_at_nodes.size() << " tensors at nodes: ";
        for (auto &p : tensors_at_nodes)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
    if (!tensors_at_elems.empty())
    {
        out << tensors_at_elems.size() << " tensors at elems: ";
        for (auto &p : tensors_at_elems)
        {
            out << "\t'" << p.first << "' (size=" << p.second->size() << ")\n";
        }
    }
}
