/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WTAG_H
#define WTAG_H

#include "tbox.h"
#include "wObject.h"
#include <string>
#include <vector>

namespace tbox
{

/**
 * @brief a Tag is a named group of 1D, 2D or 3D elements
 *
 *   The label related to:
 *      - the underlying geometrical entity of the elements
 *      - a (named) "physical entity"
 *      - a mesh partition
 * @authors Romain Boman
 */

class TBOX_API Tag : public fwk::wObject
{
public:
    int no;                       ///< label
    int dim;                      ///< dimension (1, 2 or 3 for 1D, 2D or 3D -  see .msh)
    std::string name;             ///< name given in gmsh
    std::vector<Element *> elems; ///< list of elements with this tag

    Tag(int _no, std::string const &_name = Tag::defaultName(), int _dim = 0) : wObject(), no(_no), dim(_dim), name(_name) {}
    static const std::string &defaultName()
    {
        static const std::string name("---");
        return name;
    }

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace tbox

#endif //WTAG_H
